/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.muse.core.routing.MessageHandler;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 * 
 * AbstractCapability provides a foundation for user-defined capabilities 
 * by allowing users to focus on their operations and capability-specific 
 * logic without having to worry about maintaining Muse components and 
 * other plumbing work.
 * <br><br>
 * The implementations of the initialization and shutdown routines are mostly 
 * no-ops. Nevertheless, if these methods are overridden by concrete capability 
 * classes, the implementations should be sure to call the parent class version 
 * (via 'super') before executing their own logic; for example, a new version 
 * of the initialize() method should always start with a call to super.initialize(). 
 * As the class hierarchy grows, this ensures that startup and shutdown tasks 
 * are always performed in the proper order.
 * 
 * @author Dan Jemiolo (danj)
 *
 */

public abstract class AbstractCapability implements Capability
{
    //
    // the unique URI that defines the capability within a resource type
    //
    private String _capabilityURI = null;
    
    private Environment _environment = null;
    
    //
    // WS-A Action -> MessageHandler - the MessageHandler converts 
    // SOAP -> Java and vice versa
    //
    private Map _handlersByAction = new HashMap();
    
    private boolean _hasBeenInitialized = false;
    
    private boolean _hasBeenShutdown = false;
    
    private Logger _log = null;
    
    //
    // name-value pairs specified with <init-param/>
    //
    private Map _parameters = null;
    
    //
    // optional persistence mechanism for saving/re-loading state
    //
    private Persistence _persistence = null;

    //
    // The resource implementing the capability
    //
    private Resource _resource = null;
    
    public Collection getActions()
    {
        return Collections.unmodifiableSet(_handlersByAction.keySet());
    }
    
    public String getCapabilityURI()
    {
        return _capabilityURI;
    }
    
    public Environment getEnvironment()
    {
        return _environment;
    }
    
    public String getInitializationParameter(String name)
    {
        return (String)getInitializationParameters().get(name);
    }
    
    public Map getInitializationParameters()
    {
        return _parameters;
    }
    
    public Logger getLog()
    {
        return _log;
    }
    
    public MessageHandler getMessageHandler(String action)
    {
        return (MessageHandler)_handlersByAction.get(action);
    }
    
    public Persistence getPersistence()
    {
        return _persistence;
    }

    public Resource getResource()
    {
        return _resource;
    }
    
    public boolean hasBeenInitialized()
    {
        return _hasBeenInitialized;
    }
    
    public boolean hasBeenShutdown()
    {
        return _hasBeenShutdown;
    }
    
    public void initialize() 
        throws SoapFault
    {
        _hasBeenInitialized = true;
    }
    
    public void initializeCompleted() 
        throws SoapFault
    {
        Persistence persistence = getPersistence();
        
        //
        // if the capability defines a persistence mechanism, we 
        // should ask it to reload any saved data
        //
        if (persistence != null)
        {
            ResourceManager manager = getResource().getResourceManager();
            persistence.setResourceManager(manager);
            persistence.reload();
        }
    }

    public void prepareShutdown() 
        throws SoapFault
    {
        //
        // no default action
        //
    }

    public void setCapabilityURI(String capabilityURI)
    {
        _capabilityURI = capabilityURI;
    }

    public void setEnvironment(Environment environment)
    {
        _environment = environment;
    }

    public void setInitializationParameters(Map parameters)
    {
        _parameters = parameters;
    }

    public void setLog(Logger log)
    {
        _log = log;
    }
    
    protected void setMessageHandler(MessageHandler handler)
    {
        _handlersByAction.put(handler.getAction(), handler);
    }
    
    public void setMessageHandlers(Collection handlers)
    {
        Iterator i = handlers.iterator();
        
        while (i.hasNext())
            setMessageHandler((MessageHandler)i.next());
    }
    
    public void setPersistence(Persistence persistence)
    {
        _persistence = persistence;
    }
    
    public void setResource(Resource resource)
    {
        _resource = resource;
    }

    public void shutdown() 
        throws SoapFault
    {
        _hasBeenShutdown = true;
    }    
}
