/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core;

import java.io.File;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.MessageHeaders;
import org.apache.muse.ws.addressing.soap.SimpleSoapClient;
import org.apache.muse.ws.addressing.soap.SoapClient;
import org.w3c.dom.Document;

/**
 * 
 * AbstractEnvironment is an abstract class that defines features related to 
 * the underlying platform as well as the endpoint's configuration. Its major 
 * responsibilities include storing request context on a per-thread basis and 
 * providing generic deployment information. Concrete isolation layers should 
 * provide concrete Environments that implement the abstract methods using 
 * the APIs of the underlying web services platform.
 * 
 * @author Dan Jemiolo (danj)
 * 
 * @see org.apache.muse.core.platform.AbstractIsolationLayer
 * 
 */

public abstract class AbstractEnvironment implements Environment
{
    private static final String FILE_SCHEME = "file";

	//
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(AbstractEnvironment.class);
    
    //
    // Used to load resources (files, etc) in the applications's classpath
    //
    private ClassLoader _classLoader = null;
    
    //
    // A generic per-thread context mechanism - each thread (which may 
    // represent a request to the resource) can have its own context data
    //
    private Map _contextByThread = new HashMap();

    //
    // The URI defined by the first request sent to the endpoint. This 
    // can be used in cases where a thread is not a request-handler but 
    // we still need an idea of what the resource type's EPR is. This is not 
    // guaranteed to be correct (since an endpoint may have multiple URIs 
    // that map to it), but it's better than nothing.
    //
    private String _defaultURI = null;
    
    //
    // The SOAP proxy used to make requests throughout the application
    //
    private SoapClient _soapClient = null;
    
    /**
     * 
     * The default constructor calls this('true').
     *
     */
    public AbstractEnvironment()
    {
        this(true);
    }
    
    /**
     * 
     * Initializes the AbstractEnvironment with its own ClassLoader.
     *
     * @param createSoapClient
     *        True if you want the SOAP client to be initialized. This requires 
     *        that you have a valid JAX-RPC SOAP implementation (not just the 
     *        interfaces) on the classpath.
     *
     */
    public AbstractEnvironment(boolean createSoapClient)
    {
        initialize(getClass().getClassLoader(), createSoapClient);
    }
    
    /**
     * 
     * This constructor calls this(classLoader, 'true').
     *
     */
    public AbstractEnvironment(ClassLoader classLoader)
    {
        this(classLoader, true);
    }
    
    /**
     * 
     * @param classLoader
     *        The ClassLoader that will be used to load all Java classes as well 
     *        as all file system resources.
     *
     * @param createSoapClient
     *        True if you want the SOAP client to be initialized. This requires 
     *        that you have a valid JAX-RPC SOAP implementation (not just the 
     *        interfaces) on the classpath.
     *
     */    
    public AbstractEnvironment(ClassLoader classLoader, boolean createSoapClient)
    {
        initialize(classLoader, createSoapClient);
    }
    
    public synchronized void addAddressingContext(MessageHeaders context)
    {
        _contextByThread.put(Thread.currentThread(), context);
    }
    
    public String createRelativePath(String originalPath, String relativePath)
    {    	
    	URI original = URI.create(originalPath);
			
		//
		// If the relative path is an absolute path 
		// then it only makes sense to return it since returning a absolute 
		// path relative to another path does not make sense
    	//
		if(isAbsolute(relativePath)) {
			return relativePath;
		}
				
        return original.resolve(relativePath).toString();
    }
    
    /**
     * Checks to see if a path is an absolute path. This can be absolute in two ways:
     * first it can be absolute in the URI sense (that is, it has a scheme compoenent such
     * as http:, file:, etc.). Second it can be absolute if the scheme component is null
     * or is the "file" scheme which means it is absolute relative to the filesystem. 
     * 
     * @param relativePath The given path to check
     * @return true if the path is absolute
     */
    private boolean isAbsolute(String path) {
    	URI uri = URI.create(path);
    	
		//
		// According to URI, a path is relative if and only if
		// it has a scheme component to it (http:, file:, etc.)
		//		
    	if(uri.isAbsolute()) {
			return true;
		}
		
		//		
		// Check the scheme of the URI created above because this is more
		// robust than checking substrings.
		//
		if(uri.getScheme() == null || uri.getScheme().equals(FILE_SCHEME)) {
			
			//
			// Use the path portion of the URI. This will allow the code
			// to handle file URIs.
			//
			File file = new File(uri.getPath());
			if(file.isAbsolute()) {
				return true;
			}
		}
		
		//
		// If none of the above worked, then this isn't an absolute path.
		//
		return false;
	}

	/**
     * 
     * This method can be overridden by concrete Environments to provide 
     * a different implementation of SoapClient for use by the application.
     * 
     * @return A concrete SoapClient. This implementation of Environment 
     *         returns a SimpleSoapClient object.
     *
     */
    protected SoapClient createSoapClient()
    {
        return new SimpleSoapClient();
    }
    
    public synchronized MessageHeaders getAddressingContext()
    {
        return (MessageHeaders)_contextByThread.get(Thread.currentThread());
    }
    
    public ClassLoader getClassLoader()
    {
        return _classLoader;
    }
    
    public URL getDataResource(String path) 
    {
        URL url = getClassLoader().getResource(path);
        
        if (url == null)
        {
            Object[] filler = { path };
            throw new RuntimeException(_MESSAGES.get("NoDataResource", filler));
        }
        
        return url;
    }
    
    public InputStream getDataResourceStream(String path) 
    {
        InputStream stream = getClassLoader().getResourceAsStream(path);
        
        if (stream == null)
        {
            Object[] filler = { path };
            throw new RuntimeException(_MESSAGES.get("NoDataResource", filler));
        }
        
        return stream;
    }
    
    /**
     * 
     * @return An EPR whose URI is the value returned by getDefaultURI().
     *
     */
    protected EndpointReference getDefaultEPR()
    {
        String defaultURI = getDefaultURI();
        
        if (defaultURI == null)
            throw new IllegalStateException(_MESSAGES.get("NoDefaultURI"));
        
        return new EndpointReference(URI.create(defaultURI));
    }
    
    
    public String getDefaultURI()
    {
        return _defaultURI;
    }
    
    /**
     * 
     * @param uriString
     * 
     * @return The URI being targeted by the current request. This URI 
     *         includes an IP address rather than a host name, because we 
     *         often use it to create EPRs/URIs for remote users based 
     *         on the results of a local request. Returning an IP means 
     *         that remote systems don't have to have a hosts lookup that 
     *         includes our endpoint's machine.
     *
     */
    protected String getDeploymentURI(String uriString)
    {
        URI uri = URI.create(uriString);
        
        String ip = null;
        
        try
        {
            ip = InetAddress.getLocalHost().getHostAddress();            
        }
        
        catch (UnknownHostException error)
        {
            throw new RuntimeException(error.getMessage(), error);
        }
        
        //
        // create new URI and s/host/IP
        try
        {
            uri = new URI(uri.getScheme(), 
                          uri.getUserInfo(), 
                          ip, 
                          uri.getPort(), 
                          uri.getPath(), 
                          uri.getQuery(), 
                          uri.getFragment());
        }
        
        catch (URISyntaxException error)
        {
            throw new RuntimeException(error.getMessage(), error);
        }
        
        return uri.toString();
    }
    
    public Document getDocument(String path) 
    {
        InputStream stream = getDataResourceStream(path);
        
        try
        {
            return XmlUtils.createDocument(stream);
        }
        
        catch (Exception error)
        {
            Object[] filler = { path, error.getMessage() };
            throw new RuntimeException(_MESSAGES.get("CreateDocFailed", filler));
        }
    }
    
    /**
     * 
     * @return The SOAP client that should be used by components wishing 
     *         to make web service calls. The default client provided by 
     *         this class is an instance of SimpleSoapClient.
     * 
     * @see SimpleSoapClient
     *
     */
    public SoapClient getSoapClient()
    {
        return _soapClient;
    }
    
    /**
     * 
     * Saves the ClassLoader and loads the SOAP client, if desired.
     *
     */
    private void initialize(ClassLoader classLoader, boolean createSoapClient)
    {
        if (classLoader == null)
            throw new NullPointerException(_MESSAGES.get("NullClassLoader"));
        
        _classLoader = classLoader;
        
        if (createSoapClient)
            _soapClient = createSoapClient();
    }
    
    public synchronized void removeAddressingContext()
    {
        _contextByThread.remove(Thread.currentThread());
    }
    
    public void setDefaultURI(String defaultURI)
    {
        _defaultURI = defaultURI;
    }
    
    /**
     * 
     * {@inheritDoc} 
     * The default SOAP client is an instance of SimpleSoapClient.
     * 
     * @param soapClient
     *        The SOAP client that should be used by components needing to 
     *        invoke other web services.
     * 
     * @see SimpleSoapClient
     *
     */
    public void setSoapClient(SoapClient soapClient)
    {
        if (soapClient == null)
            throw new NullPointerException(_MESSAGES.get("NullSoapClient"));
        
        _soapClient = soapClient;
    }
}
