/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core;

import java.io.File;
import java.io.InputStream;
import java.net.URL;

import org.w3c.dom.Document;

import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.MessageHeaders;
import org.apache.muse.ws.addressing.soap.SoapClient;

/**
 *
 * Environment defines features related to the underlying platform and the 
 * endpoint's configuration in a generic way. It allows resources to ask 
 * for data or task completion that depends on a platform-specific API 
 * without introducing platform-specific API usage into implementation code.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public interface Environment
{
    /**
     * 
     * Records the WS-Addressing data for the current request.
     *
     * @param context
     *        An object that can provide platform-specific information 
     *        and which is thread-unique or thread-safe.
     *
     */
    void addAddressingContext(MessageHeaders context);

    /**
     * 
     * Merges the two paths to create a valid version of the second path. 
     * This method should be used when you encounter a relative path in 
     * a document and must resolve it based on the path of the current file.
     *
     */
    String createRelativePath(String originalPath, String relativePath);

    /**
     * 
     * @return The WS-Addressing context for the current request.
     *
     */
    MessageHeaders getAddressingContext();

    /**
     * 
     * @return The ClassLoader used to load all classes and file system 
     *         resources.
     *
     */
    ClassLoader getClassLoader();

    /**
     * 
     * @param path
     *        A path that is relative to the environment's context root. 
     *        In J2EE applications this is the WAR root, in OSGi applications 
     *        this is the bundle's root.
     * 
     * @return The URL of the "data resource" found at the given path. If no 
     *         such resource is found, the method throws a RuntimeException.
     * 
     */
    URL getDataResource(String path);

    /**
     * 
     * @param path
     *        A path that is relative to the environment's context root. 
     *        In J2EE applications this is the WAR root, in OSGi applications 
     *        this is the bundle's root.
     * 
     * @return An open InputStream that can read the "data resource" found at 
     *         the given path. If no such resource is found, the method throws 
     *         a RuntimeException.
     * 
     */
    InputStream getDataResourceStream(String path);

    /**
     * 
     * @return The address of a resource type hosted by this application. This can 
     *         be used to gather some data about the wsa:Address of a resource, 
     *         although it is not guaranteed to be an exact match. The purpose of 
     *         this URI is to provide some address context when none is available 
     *         from an incoming request.
     *
     */
    String getDefaultURI();

    /**
     *  
     * @return The <b>copy</b> of the EPR of the application, as viewed in 
     *         the context of the current request. When multiple endpoints 
     *         are mapped to one application/service, this method will 
     *         return different EPRs from request to request. Muse uses 
     *         this feature to host different resource types (with different 
     *         endpoints) in the same application or bundle.
     *
     */
    EndpointReference getDeploymentEPR();

    /**
     * 
     * This is a convenience method that loads an InputStream using 
     * getDataResourceStream() and turns it into a DOM Document.
     * 
     * @see #getDataResourceStream(String)
     *
     */
    Document getDocument(String path);
    
    /**
     * 
     * @return The actual directory on the file system where the application 
     *         is deployed.
     *
     */
    File getRealDirectory();

    /**
     * 
     * @return The SOAP client that should be used by components wishing 
     *         to make web service calls. The default client provided by 
     *         this class is an instance of SimpleSoapClient.
     * 
     */
    SoapClient getSoapClient();

    /**
     * 
     * Removes the context object for the current request.
     *
     */
    void removeAddressingContext();

    /**
     * 
     * @param defaultURI
     *        The URI to use when some address context is needed, but there 
     *        is no request data to provide it.
     *
     */
    void setDefaultURI(String defaultURI);

    /**
     * 
     * Provides a new SOAP client for use by other components in the system.
     * 
     * @param soapClient
     *        The SOAP client that should be used by components needing to 
     *        invoke other web services.
     *
     */
    void setSoapClient(SoapClient soapClient);
}
