/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core;

import java.util.Collection;
import java.util.Iterator;

import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 * 
 * ResourceManager defines the interface for creating and storing resources 
 * that can be targeted by other resources (internal and external to a 
 * Muse-derived application). The resources' endpoint references (EPRs) are 
 * used to differentiate between resources in concrete implementations. This 
 * is the <em>implied resource pattern</em> required by WS-Addressing.
 * <br><br>
 * Note that ResourceManager does not initiate any lifecycle events - the 
 * addResource() method does not create a resource, and the removeResource() 
 * method does not destroy a resource. Other components can use the 
 * createResource() method to instantiate a resource and addResource() to make 
 * it visible to remote clients.
 * 
 * @author Dan Jemiolo (danj)
 * 
 * @see Resource#shutdown()
 * 
 */

public interface ResourceManager extends Initialization, Shutdown
{
    /**
     * 
     * @param listener
     *        A listener that will be fired whenever a resource is added 
     *        or removed from the resource. 
     *
     */
    void addListener(ResourceManagerListener listener);
    
    /**
     * 
     * Add a resource to the manager, associated with the given EPR.
     * 
     * @param epr
     *        The unique EPR identifying the resource instance.
     *        
     * @param resource
     *  
     * @throws SoapFault
     *         <ul>
     *         <li>If there is already a resource with the same EPR.</li>
     *         </ul>
     * 
     * @see #getResource(EndpointReference)
     * 
     */
    void addResource(EndpointReference epr, Resource resource)
        throws SoapFault;
    
    /**
     * 
     * @param definitions
     *        The ResourceDefinition objects that will be used to generate new 
     *        resource instances.
     * 
     */
    void addResourceDefinitions(Collection definitions);
    
    /**
     * 
     * Finds the resource type definition associated with the given context 
     * path (defined in muse.xml), instantiates an instance of the resource 
     * class, and sets the basic values (EPR, initialization parameters, etc.) 
     * and Muse components (Environment, log file, etc.) that it needs to 
     * operate.
     * <br><br>
     * The Resource object returned by this method has <b>not</b> been initialized 
     * yet. Once the Resource.initialize() method has been called, the resource 
     * should be added to the ResourceManager with the addResource() method.
     *
     * @param contextPath
     *        The context path for the resource type that is to be instantiated.
     *        This value is specified with the <em>context-path</em> element in 
     *        muse.xml.
     *        
     * @return An instance of the resource class. It will have a proper EPR, 
     *         all of its capabilities, and references to all of the basic 
     *         Muse components (Environment, log file, etc.). Calling code may 
     *         further modify the resource before calling initialize() and 
     *         making it available to clients.
     * 
     * @throws SoapFault
     *
     */
    Resource createResource(String contextPath)
        throws SoapFault;
    
    /**
     * 
     * @return The manager's access point for things such as the 
     *         WS-Addressing context, file system resources, and more.
     *
     */
    Environment getEnvironment();
    
    /**
     *
     * @return The number of resource instances stored by the manager.
     *
     */
    int getNumberOfResources();
    
    /**
     * 
     * @param epr
     *        The unique EPR that is associated with the desired resource.
     * 
     * @return The resource associated with the given EPR, or null if no 
     *         such resource exists.
     *
     */
    Resource getResource(EndpointReference epr);
    
    /**
     * 
     * This method allows you to find a resource type's endpoint URI given 
     * the Java interface or concrete class of one of its capabilities.
     * 
     * @param capabilityClass
     *        An interface or concrete class for one of the capabilities in 
     *        the application's resource types. This value may be defined in 
     *        <em>resource-type/capability/java-capability-class</em> in muse.xml.
     * 
     * @return A unique URL for the resource type that is partially-defined by 
     *         the given capability's interface or class, or null if no such 
     *         type exists.
     *
     */
    String getResourceContextPath(Class capabilityClass);
    
    /**
     * 
     * @return A collection containing the context paths for all resource 
     *         types specified in muse.xml. The collection may be empty.
     *
     */
    Collection getResourceContextPaths();

    /**
     * 
     * This method is just like getResourceContextPath(Class) except that 
     * it returns multiple endpoints that use the capability.
     *
     */
    Collection getResourceContextPaths(Class capabilityClass);
    
    /**
     *
     * @return An iterator over the collection of resource EPRs.
     *
     */
    Iterator getResourceEPRs();
    
    /**
     * 
     * This method allows you to search for the EPRs for all instances 
     * of a given resource type (where each resource type has a unique 
     * context path).
     * 
     * @param contextPath
     * 
     * @return An iterator over the collection of resource EPRs 
     *         whose wsa:Address includes the given context path. 
     *
     */
    Iterator getResourceEPRs(String contextPath);
    
    /**
     * 
     * @param contextPath
     * 
     * @return True if the resource type was specified with the 
     *         <em>use-router-persistence</em> flag set to 'true' in muse.xml.
     *
     */
    boolean isUsingPersistence(String contextPath);
    
    /**
     * 
     * Stops the given listener from receiving notifications about the 
     * addition and removal of resource instances.
     *
     * @param listener
     *
     */
    void removeListener(ResourceManagerListener listener);

    /**
     * 
     * Removes the EPR-resource pair from the manager. This does <b>not</b> 
     * destroy the resource - it merely removes it from the external view.
     *
     * @param epr
     *        The unique EPR that maps to the resource being removed.
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If the EPR does not map to a resource.</li>
     *         </ul>
     *
     */
    void removeResource(EndpointReference epr)
        throws SoapFault;

    /**
     * 
     * Removes each ResourceDefinition in the given Collection. After this method 
     * has returned, the application will no longer service requests for the 
     * given resource types. It will be as though they were never deployed.
     * 
     * @param definitions
     *
     */
    void removeResourceDefinitions(Collection definitions);

    void setEnvironment(Environment env);
}
