/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core;

import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 *
 * ResourceManagerListener is an interface for components that want to 
 * receive internal notifications about the addition/removal of resource 
 * instances in the Muse ResourceManager. The addition and removal of 
 * resources from the manager usually happens directly after the creation 
 * and destruction of the resources, respectively. Listening for modifications 
 * to the manager's collection of live resources allows internal components to 
 * act on these events without using some kind of non-standard public API to 
 * make them available.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public interface ResourceManagerListener
{
    /**
     * 
     * @param epr
     *        The unique EPR that is being associated with the new resource. 
     *        This value should always be used over Resource.getEndpointReference() 
     *        because some applications may choose to map multiple EPRs to the 
     *        same resource instance, and thus getEndpointReference() will not 
     *        provide an accurate value.
     *        
     * @param resource
     *        The resource instance being associated with the given EPR.
     *         
     * @throws SoapFault
     *
     */
    void resourceAdded(EndpointReference epr, Resource resource)
        throws SoapFault;
    
    /**
     * 
     * @param epr
     *        The unique EPR of the resource that has been destroyed - the 
     *        Resource object is not provided because it would most likely 
     *        be in an inconsistent state. At this point, the resource has 
     *        already been 'destroyed'.
     * 
     * @throws SoapFault
     *
     */
    void resourceRemoved(EndpointReference epr)
        throws SoapFault;
}
