/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core.descriptor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.core.Environment;
import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 * 
 * @author Dan Jemiolo (danj)
 *
 */

public class SimpleDeploymentDescriptor implements DeploymentDescriptor
{    
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = 
        MessagesFactory.get(SimpleDeploymentDescriptor.class);
    
    //
    // The resource type definitions in muse.xml
    //
    private Collection _resourceDefinitions = Collections.EMPTY_LIST;
    
    //
    // The data used to instantiate the router
    //
    private RouterDefinition _routerDefinition = null;
    
    //
    // The custom serializers defined by the user
    //
    private Collection _serializerDefinitions = Collections.EMPTY_LIST;
    
    protected Collection createResourceDefinitions(Element xml, Environment env)
        throws SoapFault
    {
        Element[] resourceXML = XmlUtils.getElements(xml, DescriptorConstants.RESOURCE_TYPE_QNAME);
        
        Collection definitions = new ArrayList(resourceXML.length);
        
        for (int n = 0; n < resourceXML.length; ++n)
        {
            ResourceDescriptor rd = createResourceDescriptor();
            rd.load(resourceXML[n], env);
            
            ResourceDefinition definition = rd.getResourceDefinition();
            definitions.add(definition);
        }
        
        return definitions;
    }
    
    protected ResourceDescriptor createResourceDescriptor()
    {
        return new SimpleResourceDescriptor();
    }
    
    protected RouterDefinition createRouterDefinition(Element xml, Environment env)
        throws SoapFault
    {
        Element routerXML = XmlUtils.getElement(xml, DescriptorConstants.ROUTER_QNAME);
        
        RouterDescriptor rd = createRouterDescriptor();
        rd.load(routerXML, env);
        
        return rd.getRouterDefinition();
    }
    
    protected RouterDescriptor createRouterDescriptor()
    {
        return new SimpleRouterDescriptor();
    }
    
    public Collection createSerializerDefinitions(Element xml, Environment env)
        throws SoapFault
    {
        QName qname = DescriptorConstants.CUSTOM_SERIALIZER_QNAME;
        Element[] serializerXML = XmlUtils.getElements(xml, qname);
        
        Collection definitions = new ArrayList(serializerXML.length);
        
        for (int n = 0; n < serializerXML.length; ++n)
        {
            SerializerDescriptor sd = createSerializerDescriptor();
            sd.load(serializerXML[n], env);
            
            SerializerDefinition definition = sd.getSerializerDefinition();
            definitions.add(definition);
        }
        
        return definitions;
    }
    
    protected SerializerDescriptor createSerializerDescriptor()
    {
        return new SimpleSerializerDescriptor();
    }
    
    public Collection getResourceDefinitions()
    {
        return _resourceDefinitions;
    }
    
    public RouterDefinition getRouterDefinition()
    {
        return _routerDefinition;
    }
    
    public Collection getSerializerDefinitions()
    {
        return _serializerDefinitions;
    }
    
    public void load(Document xml, Environment environment)
        throws SoapFault
    {
        Element root = XmlUtils.getFirstElement(xml);
        QName rootName = XmlUtils.getElementQName(root);
        
        //
        // make sure we're starting with a valid muse.xml
        //
        if (!rootName.equals(DescriptorConstants.MUSE_QNAME))
        {
            Object[] filler = { DescriptorConstants.MUSE_QNAME, rootName };
            throw new RuntimeException(_MESSAGES.get("InvalidDescriptorRoot", filler));
        }
                
        //
        // any user-defined serializers must be loaded before we try to 
        // load the operations/handlers
        //
        _serializerDefinitions = createSerializerDefinitions(root, environment);
        
        //
        // one router used to map all resource types...
        //
        _routerDefinition= createRouterDefinition(root, environment);
        
        //
        // there will be one definition for each resource type
        //
        _resourceDefinitions = createResourceDefinitions(root, environment);
        
        _routerDefinition.setResourceDefinitions(_resourceDefinitions);
    }

    public void setResourceDefinitions(Collection definitions)
    {
        _resourceDefinitions = definitions;
    }

    public void setRouterDefinition(RouterDefinition definition)
    {
        _routerDefinition = definition;
    }

    public void setSerializerDefinitions(Collection definitions)
    {
        _serializerDefinitions = definitions;
    }

    public Element toXML()
    {
        return toXML(XmlUtils.createDocument());
    }

    public Element toXML(Document doc)
    {
        RouterDefinition routerDef = getRouterDefinition();
        Collection resourceDefs = getResourceDefinitions();
        Collection serializerDefs = getSerializerDefinitions();
        
        Element root = XmlUtils.createElement(doc, DescriptorConstants.MUSE_QNAME);
        
        RouterDescriptor router = createRouterDescriptor();
        router.setRouterDefinition(routerDef);
        Element routerXML = router.toXML(doc);
        root.appendChild(routerXML);
        
        Iterator i = resourceDefs.iterator();
        ResourceDescriptor resource = createResourceDescriptor();
        
        while (i.hasNext())
        {
            ResourceDefinition next = (ResourceDefinition)i.next();
            resource.setResourceDefinition(next);
            Element resourceXML = resource.toXML(doc);
            root.appendChild(resourceXML);
        }
        
        i = serializerDefs.iterator();
        SerializerDescriptor serializer = createSerializerDescriptor();
        
        while (i.hasNext())
        {
            SerializerDefinition next = (SerializerDefinition)i.next();
            serializer.setSerializerDefinition(next);
            Element serializerXML = serializer.toXML(doc);
            root.appendChild(serializerXML);
        }
        
        return root;
    }
}
