/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core.descriptor;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.core.Environment;
import org.apache.muse.core.serializer.Serializer;
import org.apache.muse.util.ReflectUtils;
import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;

/**
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class SimpleSerializerDescriptor implements SerializerDescriptor
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = 
        MessagesFactory.get(SimpleSerializerDescriptor.class);
    
    private SerializerDefinition _definition = null;
    
    protected Class createSerializableType(Element xml, Environment env)
    {
        String className = XmlUtils.getElementText(xml, DescriptorConstants.JAVA_SERIALIZABLE_QNAME);
        
        if (className == null)
            throw new RuntimeException(_MESSAGES.get("NoSerializableClass"));
        
        return ReflectUtils.getClass(className, env.getClassLoader());
    }
    
    protected Class createSerializerClass(Element xml, Environment env)
    {
        String serializerName = XmlUtils.getElementText(xml, DescriptorConstants.JAVA_SERIALIZER_QNAME);
        
        if (serializerName == null)
            throw new RuntimeException(_MESSAGES.get("NoSerializerClass"));
        
        ClassLoader loader = env.getClassLoader();
        Class serializerClass = ReflectUtils.getClass(serializerName, loader);
        
        if (!Serializer.class.isAssignableFrom(serializerClass))
        {
            Object[] filler = { serializerName, Serializer.class.getName() };
            throw new RuntimeException(_MESSAGES.get("NotSerializer", filler));
        }
        
        return serializerClass;
    }
    
    public SerializerDefinition getSerializerDefinition()
    {
        return _definition;
    }
    
    public void load(Element xml, Environment environment)
    {
        //
        // schema type is optional, so this may be null
        //
        QName xsdType = XmlUtils.getQNameFromChild(xml, DescriptorConstants.XSD_SERIALIZABLE_QNAME);
        
        Class serializableType = createSerializableType(xml, environment);
        Class serializerClass = createSerializerClass(xml, environment);
        
        _definition = new SerializerDefinition();
        _definition.setEnvironment(environment);
        _definition.setSchemaType(xsdType);
        _definition.setSerializableType(serializableType);
        _definition.setSerializerClass(serializerClass);
    }

    public void setSerializerDefinition(SerializerDefinition definition)
    {
        _definition = definition;
    }

    public Element toXML()
    {
        return toXML(XmlUtils.createDocument());
    }

    public Element toXML(Document doc)
    {
        SerializerDefinition def = getSerializerDefinition();
        
        Element root = XmlUtils.createElement(doc, DescriptorConstants.CUSTOM_SERIALIZER_QNAME);
        XmlUtils.setElement(root, DescriptorConstants.JAVA_SERIALIZABLE_QNAME, def.getSerializableType().getName());
        XmlUtils.setElement(root, DescriptorConstants.JAVA_SERIALIZER_QNAME, def.getSerializerClass().getName());
        return root;
    }
}
