/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core.proxy;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 *
 * ProxyHandler defines the methods needed to serialize and deserialize the 
 * parameters an responses when making a remote web services invocation. The 
 * interface requires details about the target web service operation, all of 
 * which can be found in the service's WSDL. This information will allow the 
 * handler to construct appropriate SOAP messages after serializing request 
 * parameters.
 * <br><br>
 * This interface is used for <b>sending</b> outgoing messages on the 
 * client-side. <b>Receiving</b> web service requests is handled by 
 * {@linkplain org.apache.muse.core.routing.MessageHandler MessageHandler}.
 * 
 * @author Dan Jemiolo (danj)
 *
 */

public interface ProxyHandler
{
    /**
     * 
     * Deserializes the given DOM Element into a POJO that can be returned 
     * to the invoker of the web service operation. Implementations should 
     * use Muse's registered Serializers in order to deserialize the XML 
     * into objects of the proper types.
     *
     * @param xml
     *        The contents of the response SOAP Body, where the root element 
     *        contains either a simple value or a single child element that 
     *        defines a complex type. Multiple return values will not be 
     *        parsed - they must be part of a single data structure.
     * 
     * @return The return value from the operation. This object will have the 
     *         same type as getReturnType().
     *  
     * @throws SoapFault
     *         <ul>
     *         <li>If the name of the element does not match the value of 
     *         getResponseName().</li>
     *         <li>If no serialization mechanism exists for the return 
     *         value type.</li>
     *         <li>If the XML could not be deserialized into an object.</li>
     *         </ul>
     * 
     * @see #getReturnType()
     * 
     * @see org.apache.muse.core.serializer.Serializer#fromXML(Element)
     *
     */
    Object fromXML(Element xml) 
        throws SoapFault;
    
    /**
     * 
     * @return The WS-Addressing Action URI for the target operation.
     *
     */
    String getAction();
    
    /**
     *
     * @return The QName of the SOAP request element that deserializable 
     *         DOM Elements will be wrapped in.
     *
     */
    QName getRequestName();
    
    /**
     * 
     * @return The QNames for the individual parameters in the SOAP request 
     *         message, in the order they are defined by the target operation. 
     *         These are defined in the type (schema) definition of the WSDL 
     *         input message.
     *         <br><br>
     *         If the operation takes no parameters (void), or if it takes 
     *         one simple parameter that is wrapped in the root request 
     *         element (i.e., WS-RP GetResourceProperty), then this method 
     *         returns an empty array.
     *
     */
    QName[] getRequestParameterNames();
    
    /**
     * 
     * @return The XSD types that define the operation's parameters. 
     *         This is an <b>optional</b> method, and if the type data 
     *         is not provided, the returned array will be empty.
     *
     */
    QName[] getRequestParameterSchemaTypes();
    
    /**
     * 
     * @return The Java types that are expected by the serialization 
     *         framework for each of the operation's parameters. This is 
     *         an <b>optional</b> method, and if the type data is not 
     *         provided, the returned array will be empty.
     *
     */
    Class[] getRequestParameterTypes();
    
    /**
     *
     * @return The QName of the SOAP response element that deserializable 
     *         DOM Elements will be wrapped in.
     *
     */
    QName getResponseName();
    
    /**
     * 
     * @return The XSD type that defines the response value; if the 
     *         operation has no response value, this method returns null.
     *
     */
    QName getReturnSchemaType();
    
    /**
     * 
     * @return The type to which the response value should be deserialized. 
     *         If the operation has no response value, this method returns 
     *         Void.TYPE (<b>not null</b>).
     *
     */
    Class getReturnType();
    
    /**
     * 
     * @param action
     *        The WS-Addressing Action URI for the target operation. This 
     *        cannot be null.
     *
     */
    void setAction(String action);
    
    /**
     * 
     * @param requestName
     *        The QName of the SOAP request element that deserializable 
     *        DOM Elements will be wrapped in.
     *
     */
    void setRequestName(QName requestName);
    
    /**
     * 
     * @param parameterNames
     *        The QNames for the individual parameters in the SOAP request 
     *        message, in the order they are defined by the target operation. 
     *        These are defined in the type (schema) definition of the WSDL 
     *        input message. This array cannot be null, but it can be empty.
     *
     */
    void setRequestParameterNames(QName[] parameterNames);
    
    /**
     * 
     * This is an <b>optional</b> method. If the type data is not provided, 
     * getRequestParameterSchema Types() will simply return an empty array. 
     *
     * @param parameterTypes
     *        The XSD types that define the operation's parameters.
     *
     */
    void setRequestParameterSchemaTypes(QName[] parameterTypes);
    
    /**
     * 
     * This is an <b>optional</b> method. If the type data is not provided, 
     * getRequestParameterTypes() will simply return an empty array. 
     *
     * @param parameterTypes
     *        The Java types that are expected by the serialization 
     *        framework for each of the operation's parameters.
     *
     */
    void setRequestParameterTypes(Class[] parameterTypes);
    
    /**
     * 
     * @param responseName
     *        The QName of the SOAP response element that deserializable 
     *        DOM Elements will be wrapped in.
     *
     */
    void setResponseName(QName responseName);
    
    /**
     * 
     * @param returnType
     *        The XSD type that defines the response value. If the operation 
     *        has no response value, the parameter should be null (as 
     *        opposed to xsd:any).
     *
     */
    void setReturnSchemaType(QName returnType);
    
    /**
     * 
     * @param returnType
     *        The type to which the response value should be deserialized. 
     *        If the operation has no response value, the parameter can be 
     *        null or Void.TYPE (null will be translated as Void.TYPE).
     *
     */
    void setReturnType(Class returnType);
    
    /**
     * 
     * Serializes the given objects into a DOM Element that can used in a 
     * request SOAP Body. Implementations should use Muse's registered 
     * Serializers in order to serialize the objects into the proper formats.
     *
     * @param parameters
     *        The parameters for the operation, which must be serialized 
     *        into XML.
     * 
     * @return The XML equivalent of the given objects, wrapped in an element 
     *         whose QName is the value of getRequestName(). Each object's XML 
     *         fragment will be wrapped in the equivalent QName from the array 
     *         returned by getRequestParameterNames().
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If no serialization mechanism exists for an object's type.</li>
     *         <li>If an object could not be serialized into XML.</li>
     *         </ul>
     * 
     * @see #getRequestName()
     * @see #getRequestParameterNames()
     * 
     * @see org.apache.muse.core.serializer.Serializer#toXML(Object, QName)
     * 
     */
    Element toXML(Object[] parameters) 
        throws SoapFault;
}
