/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core.serializer;

import java.lang.reflect.Array;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.util.ReflectUtils;
import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 *
 * ArraySerializer is a wrapper class that transforms arrays of any 
 * <em>registered</em> type into XML and back. It uses the Serializer for 
 * a given type in order to process individual elements in an array and 
 * provides a standard format for arrays over SOAP. Note that this is not 
 * the only way to send arrays over SOAP - SOAP 1.1 and SOAP 1.2 both 
 * define (incompatible) methods for serializing arrays. This class does 
 * not prevent use of these formats, it simply defines a simple system that 
 * allows Muse users to have their arrays transformed automatically.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class ArraySerializer implements Serializer
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(ArraySerializer.class);
    
    //
    // The Class that represents the type AS AN ARRAY. This can be obtained 
    // from any type as MyType[].class
    //
    private Class _arrayClass = null;
    
    //
    // The serializer for the non-array version of the Class
    //
    private Serializer _classSerializer = null;
    
    /**
     * 
     * @param arrayClass
     *        The Class that represents the type <b>as an array</b>. This 
     *        can be obtained for any type using <em>MyType[].class</em> 
     *        or by invoking the getClass() method on an array object.
     * 
     * @param classSerializer
     *        The serializer for the non-array version of the Class. This 
     *        will be used to serialize the individual items in the arrays.
     *
     */
    public ArraySerializer(Class arrayClass, Serializer classSerializer)
    {
        if (arrayClass == null)
            throw new NullPointerException(_MESSAGES.get("NullArrayClass"));
        
        if (classSerializer == null)
            throw new NullPointerException(_MESSAGES.get("NullSerializer"));
        
        if (!arrayClass.isArray())
        {
            Object[] filler = { arrayClass.getName() };
            throw new RuntimeException(_MESSAGES.get("NotArrayClass", filler));
        }
        
        _arrayClass = arrayClass;
        _classSerializer = classSerializer;
    }
    
    /**
     * 
     * {@inheritDoc}
     * <br><br>
     * Arrays that are deserialized <em>from</em> XML will have one object 
     * for each child element in a DOM tree:
     * <br><br>
     * <code>
     * &lt;UserIDArray&gt;<br>
     * &nbsp;&nbsp;&nbsp;&lt;!-- each child is an item in the array --&gt;<br>
     * &nbsp;&nbsp;&nbsp;&lt;UserID&gt;danj&lt;/UserID&gt;<br>
     * &nbsp;&nbsp;&nbsp;&lt;UserID&gt;snible&lt;/UserID&gt;<br>
     * &nbsp;&nbsp;&nbsp;&lt;UserID&gt;bsubram&lt;/UserID&gt;<br>
     * &lt;/UserIDArray&gt;<br>
     * </code>
     * 
     */
    public Object fromXML(Element xml) 
        throws SoapFault
    {
        Element[] children = XmlUtils.getAllElements(xml);
        
        Class type = ReflectUtils.getClassFromArrayClass(getArrayClass());
        Object array = Array.newInstance(type, children.length);
        Serializer serializer = getClassSerializer();
        
        for (int n = 0; n < children.length; ++n)
        {
            Object result = serializer.fromXML(children[n]);
            Array.set(array, n, result);
        }
        
        return array;
    }
    
    public Class getArrayClass()
    {
        return _arrayClass;
    }
    
    public Serializer getClassSerializer()
    {
        return _classSerializer;
    }
    
    public Class getSerializableType()
    {
        return getArrayClass();
    }
    
    /**
     * 
     * {@inheritDoc}
     * <br><br>
     * Arrays that are serialized <em>to</em> XML will be encapsulated with 
     * elements shown below (element names are actual names):
     * <br><br>
     * <code>
     * &lt;YourTypeName&gt;<br>
     * &nbsp;&nbsp;&nbsp;&lt;YourTypeName&gt;value #1&lt;/YourTypeName&gt;<br>
     * &nbsp;&nbsp;&nbsp;&lt;YourTypeName&gt;value #2&lt;/YourTypeName&gt;<br>
     * &nbsp;&nbsp;&nbsp;&lt;YourTypeName&gt;value #3&lt;/YourTypeName&gt;<br>
     * &lt;/YourTypeName&gt;<br>
     * </code>
     * 
     */
    public Element toXML(Object obj, QName qname)
        throws SoapFault
    {
        Serializer serializer = getClassSerializer();
        Element root = XmlUtils.createElement(qname);
        
        int length = Array.getLength(obj);
        
        //
        // then add each item, wrapped in an <ArrayItem> tag
        //
        for (int n = 0; n < length; ++n)
        {
            Object item = Array.get(obj, n);
            Element child = serializer.toXML(item, qname);            
            root.appendChild(child);
        }
        
        return root;
    }    
}
