/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core.serializer;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 *
 * Serializer is a simple interface for serializing and deserializing 
 * individual objects to and from XML. It is different from the 
 * {@linkplain org.apache.muse.core.routing.MessageHandler MessageHandler} 
 * class because it is unaware of SOAP messages - it can only deal with one 
 * object at a time. This interface should be used to provide serialization 
 * for user-defined types that are exposed in web service parameter lists 
 * or return types.
 * <br><br>
 * New Serializers can be introduced to Muse by adding the following XML 
 * to the Muse deployment descriptor (muse.xml):
 * <br><br>
 * <code>
 * &lt;custom-serializer&gt;<br>
 * &nbsp;&nbsp;&nbsp;&lt;java-serializable-type&gt;<b>the new type</b>&lt;/java-concrete-class&gt;<br>
 * &nbsp;&nbsp;&nbsp;&lt;java-serializer-class&gt;<b>the type's serializer</b>&lt;/java-serializer-class&gt;<br>
 * &lt;/custom-serializer&gt;<br><br>
 * </code>
 * This package contains many Serializers for built-in types, including 
 * primitives and simple types. These built-in types have their Serializers 
 * loaded at startup by Muse.
 * 
 * @author Dan Jemiolo (danj)
 *
 */

public interface Serializer
{
    /**
     * 
     * Deserializes the DOM Element into a POJO whose type this Serializer 
     * is responsible for.
     *
     * @param xml
     *        The XML representation of a Java object. If the type is 
     *        primitive or simple (not a primitive, but serialized to a 
     *        string), the value should be a DOM Text node inside the 
     *        given Element.
     * 
     * @return A POJO constructed from the given XML. If the Element is 
     *         empty <b>and</b> the Serializer's type is a non-primitive, 
     *         the method will return null.
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If the XML is not a valid representation of the 
     *         Serializer's type.</li>
     *         </ul>
     *
     */
    Object fromXML(Element xml)
        throws SoapFault;
    
    Class getSerializableType();
    
    /**
     * 
     * Serializes the given object into a DOM Element, the value wrapped 
     * inside a root element with the given name.
     *
     * @param obj
     *        The object to serialize into XML, or null.
     * 
     * @param qname
     *        The name of the element which the XML representation will be 
     *        wrapped in.
     * 
     * @return An Element whose name is the one given, and whose child Nodes 
     *         represent the object. If the object was null, an empty Element 
     *         is returned.
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If the object could not be serialized to XML.</li>
     *         </ul>
     *
     */
    Element toXML(Object obj, QName qname)
        throws SoapFault;
}
