/* 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.muse.core.serializer;

import java.net.URI;
import java.net.URL;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.util.ReflectUtils;
import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlSerializable;

/**
 * 
 * SerializerRegistry is a singleton component that keeps track of what 
 * Java types have {@linkplain Serializer Muse Serializers} for use by 
 * SOAP handlers and user code. The class will automatically register 
 * all of Muse's built-in serializers for basic types when it is 
 * instantiated; the Muse core engine will also register user-defined 
 * serializers when it finds them in the deployment descriptor. Although 
 * the deployment descriptor is the preferred way of adding a serializer 
 * to the registry, there is nothing preventing a user from doing so 
 * programmtically using the registerSerializer() method.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class SerializerRegistry
{    
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(SerializerRegistry.class);
    
    private static final SerializerRegistry _SINGLETON = new SerializerRegistry();
    
    //
    // keep instance map as a tree so that we can show a sorted list 
    // of available serializers if an error occurs (no serializer found)
    //
    private Map _serializers = new TreeMap(new ClassComparator());
    
    /**
     * 
     * @return The singleton instance of this class.
     *
     */
    public static SerializerRegistry getInstance()
    {
        return _SINGLETON;
    }
    
    private SerializerRegistry()
    {
        //
        // create and save all of the serializers for the built-in types
        //
        
        registerSerializer(Boolean.class, new BooleanSerializer());
        registerSerializer(boolean.class, new BooleanSerializer());
        registerSerializer(Double.class, new DoubleSerializer());
        registerSerializer(double.class, new DoubleSerializer());
        registerSerializer(Float.class, new FloatSerializer());
        registerSerializer(float.class, new FloatSerializer());
        registerSerializer(Integer.class, new IntegerSerializer());
        registerSerializer(int.class, new IntegerSerializer());
        registerSerializer(Long.class, new LongSerializer());
        registerSerializer(long.class, new LongSerializer());
        registerSerializer(Short.class, new ShortSerializer());
        registerSerializer(short.class, new ShortSerializer());
        registerSerializer(String.class, new StringSerializer());
        registerSerializer(URI.class, new UriSerializer());
        registerSerializer(URL.class, new UrlSerializer());
        registerSerializer(Date.class, new DateSerializer());
        registerSerializer(QName.class, new QNameSerializer());
        registerSerializer(Element.class, new ElementSerializer());
        registerSerializer(XmlSerializable.class, new XmlSerializableSerializer());
    }
    
    /**
     * 
     * @param type
     *        The Java class that will be serialized to XML (and back).
     * 
     * @return The Serializer that can handle the given Java type.
     *
     */
    public Serializer getSerializer(Class type)
    {
        if (type == null)
            throw new NullPointerException(_MESSAGES.get("NullClass"));
        
        Serializer ser = searchClassHierarchy(type);
        
        //
        // found a match - return it
        //
        if (ser != null)
            return ser;
        
        //
        // otherwise, construct an error message the has all of the 
        // available types in it
        //
        Iterator i = _serializers.keySet().iterator();
        
        StringBuffer availableTypes = new StringBuffer(1024);
        availableTypes.append('\n');
        
        while (i.hasNext())
        {
            availableTypes.append('\n');
            availableTypes.append(i.next());
        }
        
        Object[] filler = { type, availableTypes };
        throw new RuntimeException(_MESSAGES.get("NoSerializer", filler));
    }
    
    /**
     * 
     * Associates the given serializer with the type and any of its sub-types. 
     * An ArraySerializer is also created for arrays of the type.
     * 
     * @param type
     * @param ser
     *
     */
    public void registerSerializer(Class type, Serializer ser)
    {
        if (type == null)
            throw new NullPointerException(_MESSAGES.get("NullClass"));
        
        if (ser == null)
            throw new NullPointerException(_MESSAGES.get("NullSerializer"));
        
        //
        // make an array version of the serializer
        //
        Class arrayType = ReflectUtils.getArrayClassFromClass(type);
        Serializer array = new ArraySerializer(arrayType, ser);
        
        _serializers.put(type, ser);
        _serializers.put(arrayType, array);
    }
    
    /**
     * 
     * Looks through the parent class and interfaces of the given type to 
     * see if one of them has a registered serializer that could be used 
     * on the sub-type. The paren classes are considered first, then the 
     * interfaces of the type, then the interfaces of each parent class.
     * 
     * @param type
     * 
     * @return A serializer for one of the type's parent classes or interfaces, 
     *         or null if no possible serializer is found.
     *
     */
    protected Serializer searchClassHierarchy(Class type)
    {
        Serializer ser = (Serializer)_serializers.get(type);
        
        if (ser != null)
            return ser;
        
        //
        // if there's a parent class (not java.lang.Object), recurse 
        // up the hierarchy for our search
        //        
        Class parent = type.getSuperclass();
        
        if (parent != null && parent != Object.class)
        {
            ser = searchClassHierarchy(parent);
            
            if (ser != null)
                return ser;
        }
        
        //
        // start the recursion again - this time with interfaces
        //
        Class[] interfaces = type.getInterfaces();
        
        for (int n = 0; n < interfaces.length; ++n)
        {
            ser = searchClassHierarchy(interfaces[n]);
            
            if (ser != null)
                return ser;
        }
        
        return null;
    }
    
    /**
     * 
     * ClassComparator sorts java.lang.Class objects by name (ABC order).
     *
     * @author Dan Jemiolo (danj)
     *
     */
    protected class ClassComparator implements Comparator
    {
        public int compare(Object obj1, Object obj2)
        {
            Class class1 = (Class)obj1;
            Class class2 = (Class)obj2;
            return class1.getName().compareTo(class2.getName());
        }
    }
}
